"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = _default;

var _lodash = require("lodash");

var _react = require("react");

var _Random = _interopRequireDefault(require("../lib/Random"));

var _Pub = _interopRequireDefault(require("../lib/Pub"));

var _useTracker = _interopRequireDefault(require("./useTracker"));

var _Meteor = _interopRequireDefault(require("../Meteor"));

var _ejson = _interopRequireDefault(require("ejson"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function depsFromValuesOf(params) {
  if ((0, _lodash.isObject)(params)) {
    return Object.values(params);
  }

  if (Array.isArray(params)) {
    return params;
  }

  return typeof params === 'undefined' ? [] : [params];
}

function info(msg) {
  console.info("usePub: ".concat(msg));
}

function subId(name, deps, refId) {
  return _ejson["default"].stringify({
    name: name,
    deps: deps,
    refId: refId
  });
}

function _default(_ref, dependencies) {
  var name = _ref.name,
      _ref$params = _ref.params,
      params = _ref$params === void 0 ? {} : _ref$params,
      userId = _ref.userId,
      _ref$fetch = _ref.fetch,
      fetch = _ref$fetch === void 0 ? function () {
    return null;
  } : _ref$fetch;
  var allArgsSet = !Object.values(params).some(function (x) {
    return x === undefined;
  });
  var deps = dependencies || [userId !== null && userId !== void 0 ? userId : _Meteor["default"].userId()].concat(_toConsumableArray(depsFromValuesOf(params)));
  var ref = (0, _react.useRef)(null);

  if (ref.current === null && allArgsSet) {
    ref.current = {
      subs: {},
      id: _Random["default"].id()
    };

    if (_Meteor["default"].isVerbose()) {
      var p = JSON.stringify(params);
      var d = JSON.stringify(deps);
      info("New ref ".concat(name, "(").concat(p, ")").concat(d, ", refId=").concat(ref.current.id));
    }
  } // stop publications on unmount


  (0, _react.useEffect)(function () {
    return function () {
      var _ref$current, _ref$current2;

      var id = subId(name, deps, (_ref$current = ref.current) === null || _ref$current === void 0 ? void 0 : _ref$current.id);

      if ((_ref$current2 = ref.current) !== null && _ref$current2 !== void 0 && _ref$current2.subs[id]) {
        if (_Meteor["default"].isVerbose()) {
          info("Unmounting ".concat(ref.current.id, ", unsub ").concat(id));
        }

        _Pub["default"].stop(ref.current.subs[id], ref.current.id);

        delete ref.current.subs[id];
      }
    };
  }, deps);

  var formatError = function formatError(_ref2) {
    var error = _ref2.error,
        reason = _ref2.reason;
    return error ? {
      error: error,
      reason: reason
    } : null;
  };

  return (0, _useTracker["default"])(function () {
    var _ref$current$subs$id;

    if (!allArgsSet) {
      return [undefined, false, false];
    }

    var id = subId(name, deps, ref.current.id);
    var sub = (_ref$current$subs$id = ref.current.subs[id]) !== null && _ref$current$subs$id !== void 0 ? _ref$current$subs$id : _Pub["default"].subscribe(name, params, ref.current.id);

    if (!ref.current.subs[id]) {
      ref.current.subs[id] = sub;
    }

    var result = !sub.error() ? fetch() : undefined;

    if (_Meteor["default"].isVerbose()) {
      var _sub$error;

      var _p = JSON.stringify(params);

      var _d = JSON.stringify(deps);

      var r = sub.ready();
      var e = JSON.stringify(formatError((_sub$error = sub.error()) !== null && _sub$error !== void 0 ? _sub$error : {}));
      info("Ready=".concat(r, " ").concat(name, "(").concat(_p, ")").concat(_d, ", error=").concat(e, ", refId=").concat(ref.current.id));
    }

    var isLoading = !sub.error() && !sub.ready() && result !== undefined; // console.log({ name, result, isLoading, error: sub.error() });

    return [result, isLoading, sub.error()];
  }, deps);
}