"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _Data = _interopRequireDefault(require("../Data"));

var _utils = require("../lib/utils");

var _Mongo = _interopRequireDefault(require("../Mongo"));

var _Meteor = _interopRequireDefault(require("../Meteor.js"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var TOKEN_KEY = 'Meteor.loginToken';
var Users = new _Mongo["default"].Collection('users');

function info(msg) {
  console.info("User: ".concat(msg));
}

var User = {
  users: Users,
  user: function user() {
    if (!User._userIdSaved) {
      return null;
    }

    return Users.findOne(User._userIdSaved);
  },
  userId: function userId() {
    if (!User._userIdSaved) {
      return null;
    }

    var user = Users.findOne(User._userIdSaved);
    return user && user._id;
  },
  _isLoggingIn: true,
  loggingIn: function loggingIn() {
    return User._isLoggingIn;
  },
  logout: function logout(callback) {
    _Meteor["default"].call('logout', function (err) {
      User.handleLogout();

      _Meteor["default"].connect();

      typeof callback === 'function' && callback(err);
    });
  },
  handleLogout: function handleLogout() {
    localStorage.removeItem(TOKEN_KEY);
    _Data["default"]._tokenIdSaved = null;
    User._userIdSaved = null;
  },
  loginWithPassword: function loginWithPassword(selector, password, callback) {
    if (typeof selector === 'string') {
      if (selector.indexOf('@') === -1) {
        selector = {
          username: selector
        };
      } else {
        selector = {
          email: selector
        };
      }
    }

    User._startLoggingIn();

    _Meteor["default"].call('login', {
      user: selector,
      password: (0, _utils.hashPassword)(password)
    }, function (err, result) {
      User._endLoggingIn();

      User._handleLoginCallback(err, result);

      typeof callback === 'function' && callback(err);
    });
  },
  logoutOtherClients: function logoutOtherClients() {
    var callback = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : function () {};

    _Meteor["default"].call('getNewToken', function (err, res) {
      if (err) {
        return callback(err);
      }

      User._handleLoginCallback(err, res);

      _Meteor["default"].call('removeOtherTokens', function (err) {
        callback(err);
      });
    });
  },
  _login: function _login(user, callback) {
    User._startLoggingIn();

    _Meteor["default"].call('login', user, function (err, result) {
      User._endLoggingIn();

      User._handleLoginCallback(err, result);

      typeof callback === 'function' && callback(err);
    });
  },
  _startLoggingIn: function _startLoggingIn() {
    User._isLoggingIn = true;

    _Data["default"].notify('loggingIn');
  },
  _endLoggingIn: function _endLoggingIn() {
    User._isLoggingIn = false;

    _Data["default"].notify('loggingIn');
  },
  _handleLoginCallback: function _handleLoginCallback(err, result) {
    if (!err) {
      _Meteor["default"].isVerbose() && info('User._handleLoginCallback::: token:', result.token, 'id:', result.id);
      localStorage.setItem(TOKEN_KEY, result.token);
      _Data["default"]._tokenIdSaved = result.token;
      User._userIdSaved = result.id;

      _Data["default"].notify('onLogin');
    } else {
      _Meteor["default"].isVerbose() && info('User._handleLoginCallback::: error:', err);

      _Data["default"].notify('onLoginFailure');

      User.handleLogout();
    }

    _Data["default"].notify('change');
  },
  _loginWithToken: function _loginWithToken(value) {
    _Data["default"]._tokenIdSaved = value;

    if (value !== null) {
      _Meteor["default"].isVerbose() && info('User._loginWithToken::: token:', value);

      User._startLoggingIn();

      _Meteor["default"].call('login', {
        resume: value
      }, function (err, result) {
        User._endLoggingIn();

        User._handleLoginCallback(err, result);
      });
    } else {
      _Meteor["default"].isVerbose() && info('User._loginWithToken::: token is null');

      User._endLoggingIn();
    }
  },
  getAuthToken: function getAuthToken() {
    return _Data["default"]._tokenIdSaved;
  },
  _loadInitialUser: function _loadInitialUser() {
    return _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
      var value;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              value = null;
              _context.prev = 1;
              _context.next = 4;
              return localStorage.getItem(TOKEN_KEY);

            case 4:
              value = _context.sent;
              _context.next = 10;
              break;

            case 7:
              _context.prev = 7;
              _context.t0 = _context["catch"](1);
              console.warn('LocalStorage error: ' + _context.t0.message);

            case 10:
              _context.prev = 10;

              User._loginWithToken(value);

              return _context.finish(10);

            case 13:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, null, [[1, 7, 10, 13]]);
    }))();
  }
};
var _default = User;
exports["default"] = _default;